# Récupération des données boursières {-}
Le service met à disposition des utilisateurs des informations boursières de différents actifs. Pour pouvoir mettre à disposition de tels données, il faut les récupérer et les stocker. La récupération des données se fait via des sources externes mettant à disposition des (+API_a) offrant un accès à ces données. Ces API mettent à disposition des données en continue ce qui permet de récupérer l'ensemble des ordres et activités qui ont lieu sur les marchés.

## Kraken {-}
Kraken est une plateforme de trading qui propose différents services en lien avec les crypto-monnaies. Le service principal de cette plateforme est le trading en temps réel des crypto-monnaies présentes sur la plateforme (Bitcoin, Ethereum, etc.). Pour les développeurs, Kraken propose une API ainsi qu’un WebSocket permettant d'accéder directement aux fonctionnalités de leur service ainsi qu'a l'historique de données OHLC des actifs. L' (+API_a) permet entre autres de récupérer l’historique des échanges pour une paire de monnaies donnée, l’historique de ses propres transactions, etc.

L' (+API_a) de Kraken met à disposition l’historique (+OHLC_a) d'un actif de crypto-monnaies. Il est possible de choisir l’intervalle de temps qui sépare chaque donnée. Les intervalles disponibles sur le service sont 1, 5, 15, 30, 60, 240, 1440, 10080 et 21600 minutes. Il faut cependant faire attention, seules les 720 dernières entrées des actifs sont disponibles et cela est valable quel que soit l’intervalle de temps choisi. Il est donc nécessaire de régulièrement faire appel au service pour récupérer tout l’historique d’un actif sur l’intervalle de temps souhaité.

Afin d'offrir un ensemble de données complet et précis, l'intervalle de temps entre chaque l'OHLC devra être le plus petit possible. Ici, le plus petit intervalle disponible est d’une minute. Un service sera donc mis en place afin de récupérer de manière régulière l'ensemble des données.

Voici la structure de données retournée par l'API

\cimg{figs/kraken-api-ohlc.png}{scale=0.5}{Messages de retour de l'historique OHLC}{Source : kraken.com \url{https://www.kraken.com/features/api\#get-ohlc-data}, ref. URL10}

Dans ce message, on retrouve l'ensemble de la structure OHLC ainsi que le volume. Ces données subiront quelques transformations afin de correspondre à un format générique défini à la fin de ce chapitre.

L'Order Book des marchés de Kraken sera lui récupéré via WebSocket. La structure du message de retour se présente sous cette forme.

\cimg{figs/kraken-ws-ob.png}{scale=0.25}{Structure de message WebSocket pour l'Order Book Kraken}{Source : doc.kraken.com \url{https://docs.kraken.com/websockets/\#message-book}, ref. URL11}

On retrouve dans cette structure l'ensemble des Aks / Bid pour un certain prix et un certain volume. Tout comme les données OHLC, ces données subiront des modifications afin de pouvoir être stockées selon un format générique.

### Les limtes de l'API {-}
Chaque (+API_a) que l'on peut trouver sur internet possède des règles de limitations d'appel au service fournit. Les limites mises en place par le service ne sont pas contraignantes. Les règles imposées par Kraken fonctionnent de la manière suivante.

Chaque utilisateur de l'(+API_a) a un compteur, il est initialisé à 0. Pour chaque appel à l'(+API_a), ce compteur va être incrémenté d’un chiffre allant de zéro à deux. Ce compteur est décrémenté, sans abonnement, d’un toutes les trois secondes et la limite à ne pas atteindre est de 15. Si cette limite est dépassée, l’utilisateur sera bloqué pendant 15 minutes. La récupération de l'historique OHLC ne coûte qu’un par appel. Mais il faudra faire attention à ne pas dépasser la limite car l’historique n’enregistre que les 720 dernières minutes du cours. Des abonnements payants sont proposés afin d’augmenter cette limite.

## Alphavantage {-}
Alphavantage est un service mettant à disposition une (+API_a) permettant de récupérer les informations boursières de trois différents marchés
\begin{itemize}
\item Le marché des changes (Forex ou FX)
\item Le marché des actions (Stock)
\item Le marché des crypto-monnaies
\end{itemize}

A l’inverse de Kraken, Alphavantage permet par exemple de récupérer les informations de l’action TLSA (action de l’entreprise Tesla) ou encoe AAPL (action de l’entreprise Apple). Il est aussi possible de récupérer les informations du cours du franc suisse face au dollar ou du franc suisse face à l’euro. Leur (+API_a) offre même la possibilité de récupérer des indicateurs technique d'un actif donné. Alphavantage limite, tout comme Kraken, les appels à leur (+API_a).

Leurs règles de limitaion sont plus simple que celle de Kraken. Le nombre d’appels est limité à 500 par jours et à cinq par minutes. Il est cependant possible d’augmenter ces limites via un abonnement payant.
 
L’API propose deux formats de retour : le format (+JSON_a) et le format (+CSV_a). Les données OHLC sur l'intervalle de une minute sont stockées pendant sept jours. Voici un exemple de données OHLC au format CSV retourné par leur service.

```json
timestamp,open,high,low,close,volume
2020-03-24 05:52:37,8.3800,8.3800,8.3800,8.3800,0
2020-03-24 05:51:00,8.3940,8.4120,8.3720,8.3760,193393
```

On retrouve dans ce message de retour l'ensemble des données permettant de créer la structure OHLC. En complément de l'OHLC, le volume est aussi retourné.

\newpage

## Bitfinex {-}
Bitfinex est un marché d'échange de crypto-monnaies et de forex appartenant à iFinex Inc. Leur service met à disposition une API offrant un accès à leur plateforme de trading et aux informations des marchés.
Ce qui est intéressant avec Bitfinex, c'est l'historique de données. A l'inverse de Kraken et Alphavantage, il est possible de récupérer l'historique (+OHLC_a) sur une échelle de temps relativement grande tout en gardant un intervalle de temps de une minute. Il a été possible de remonter l'historique jusqu'à six mois en arrière tout en gardant la même granularité de temps entre les données OHLC.

Une API ainsi qu'un WebSocket sont mis à disposition des utilisateurs de leur service. L'API permet de récupérer l'historique de données sous la forme de tableau (+JSON_a).

```json
[
    // timestamp in ms, open, close, high, low, volume
    [1573562880000,0.0003,0.0003,0.0003,0.0003,24083.8638948],
    [1573562820000,0.00030178,0.00033105,0.00033105,0.00030178,1840.0074507599998]
]
```

## Pulling régulier des données {-}
La première étape dans le développement de ce projet, est la récupération de données boursières. En effet, comme vue dans les chapitres précédents, il est difficile d'obtenir un petit intervalle de temps séparant chacune des données OHLC. Pour pouvoir récupérer un maximum d'informations, un script de pulling permet de régulièrement faire appel aux API et de stocker les données. Ce script est exécuté de manière régulière grâce à l'utilitaire unix crontab. Crontab est un planificateur de tâches qui permet d'exécuter n'importe quelle commande shell de manière régulière.

Afin de récupérer un maximum de données et pour s'assurer de n'en perde aucune, le planificateur execute le script de pulling toutes les quatres heures. Les données sont ensuite filtrée pour éviter de stocker les doublons avant d'être stockées.

## Les données boursières en temps réel {-}
Parmis les trois plateformes de trading vu précédement, seules deux possèdent un accès au service via Websocket. Ce service est particulièrement intéressant car il permet de trader et d'accèder aux informations des marchés en temps réel. Les WebSockets sont directements connecter au service de stockage et récupèrent les données OHLC ainsi que l'Order Book.

Les deux plateformes proposant un accès à leur service via websocket sont Karken et Bitfinex. Toutes deux offrent la possibilité de récupérer l'Order Book et les données (+OHLC_a) en temps réel.

## Synthèse {-}
Bien que ces API aient des différences notables dans la manière de structurer les données, elles ont en commun une seule et même structure : OHLC. De là, il est facile de définir un format de représentation standard commun à tout le service. Les données sont donc représentées au format CSV de la manière suivante :

\begin{center}
`actif;timestamp;open;high;low;close;volume`
\end{center}

Les quatre données de l’OHLC ainsi que le volume et le nom de l’actif sont au format string. Le timestamp quant à lui sera au format Unix timestamp pour plus de simplicité et pour éviter toute erreur de parsing.

Quant aux données de l'order book, une structure au format CSV similaire à celle de l'OHLC a été définis.
\begin{center}
`asset:timestamp:[a | b]:price:volume`
\end{center}

On retrouve dans cette structure les propositions de Ask ou Bid, le prix ainsi que le volume. Le troisième élément permet d'indiquer si il s'agit d'un Ask ou d'un Bid.

La structure ayant été définie, il est maintenant possible de récupérer et stocker les données (+API_a). Il faut cependant déployer un système de stockage performant permettant le traitement de grandes quantités d'informations.

\pagebreak