# Stockage des données boursières {-}
Le logiciel utilisé afin de stocker et distribuer les données devra est capable de traiter, envoyer, et stocker de manière persistante de très grandes quantités de données en des temps très faibles.

## Présentation de Kafka {-}
Apache Kafka est une plateforme de streaming distribuée développé en Scala par LinkedIn avant d'être donnée à la Apache Software Foundation. Cette plateforme a la capacité d'être déployée en cluster sur une ou plusieurs machines à travers un ou plusieurs datacenter. Ceci offre à Kafka une grande tolérence aux pannes et de réplications des données. Il est capable de traiter des flux de données extrêmement grands sans avec une très faible latence.
Voici quelque bénéfices de l'utilisation de kafka :

\begin{itemize}
\setlength\itemsep{-0.6em}
\item Tolérance aux pannes
\item Faible latence
\item Persistance
\end{itemize}

Kafka met à disposition des développeurs cinq API permettant d'accéder au cluster afin de récupérer, produire des messages, connecter un service ou encore administrer administrer le cluster.

\begin{itemize}
\setlength\itemsep{-0.6em}
\item Consumer API
\item Producer API
\item Stream API
\item Connect API
\item Admin API
\end{itemize}

Toutes ces API permettent au développeur de travailler aisément autour du cluster et de ses fonctionnalités.

## Fonctionnement de Kafka {-}
Kafka est un message-broker de type publish-subscribe, il a donc la possibilité de recevoir, envoyer ainsi que stocker des données. Pour ce faire, Kafka est basé sur le principe de Topics et de Logs. Les topics sont des catégories dans lesquelles les données sont stockées. Ces topics sont, au sein du cluster, distribués sur chaque noeud que l'on appelle Broker. Il est généralement déployé sur plusieurs machines distinctes ou encore dans des (+VM_a).

### Zookeeper {-}
Avant de rentrer dans les détails de fonctionnement de Kafka, il faut tout d'abord comprendre ce qu'est Zookeeper. Zookeeper est aussi un logiciel de la Apache Software Foundation, c'est un service de gestion de configuration pour système distribué. Il est indispensable au bon fonctionnement d'un cluster Kafka car il permet de garder un oeil sur chacun des noeuds et sur toutes les partitions de celui-ci. Un cluster Kafka ne peut pas fonctionner sans Zookeeper.

Lors du déploiement d'un cluster Kafka, Zookeeper est le premier logiciel qu'il faut configurer. Cette configuration doit se faire avec le plus grand soin car sans lui le cluster ne fonctionne pas. Il doit avoir connaissance de toutes les adresses IP des machines afin de synchroniser et élire les partitions leader de chaque topic. Chaque instance Zookeeper est identifiable à l'aide d'un identifiant unique.

\newpage

### Topics, logs et partitions {-}
Kafka organise les données sous forme de topic. Un topic est une catégorie / rubrique dans laquelle les données y sont enregistrées. Il est identifié à l’aide d’un nom unique et il peut y en avoir autant que l’on souhaite. A titre de comparaison, un topic est semblable à la table d’une base de données. Toutes les données relatant un sujet y sont enregistrées.

\cimg{figs/topics.png}{scale=0.5}{Exemple d'un topic Kafka au sein d'un seul broker}{Source : Réalisé par Paschoud Nicolas}

Chaque topic est constitué d’une ou plusieurs partitions, chacune distribuée à travers les brokers du cluster. Les données sont publiées dans un topic mais distribuées de manière aléatoire entre les partitions. L'ordre des messages dans la partition est immutable, chaque message se suit dans l'ordre de publication. Il est cependant possible d’ajouter une clé au message afin de les rassembler dans une partition spécifique. L’enregistrement dans les partitions à l’aide d’une même clé se fait de manière transparente par Kafka. Le cluster maintiens un fichier log pour chacune des partitions d'un topic dans lequel les données sont enregistrées, ce fichier log se trouve sur le disque de chacun des broker du cluster.

\cimg{figs/topics-partitions.png}{scale=0.5}{Exemple d'un topic Kafka avec trois partitions}{Source : Réalisé par Paschoud Nicolas}

Chaque message publié dans le cluster se voit attribuer un identifiant unique appelé offset (celui-ci est différent de la clé fournie pour s’assurer que les messages se trouvent dans une même partition). L’offset est unique et n’est attribué qu’une seule fois. Il n'est jamais associé une seconde fois à un message même si une suppression a lieu. Il est unique et spécifique à une partition, il ne fait sens qu’au sein de celle-ci. L’offset trois de la partition zéro n’est pas le même dans la partition une.

Il est important de noté que l'ordre de lecture des messages au sein d'une partition est garanti mais il ne l'est pas entre les partitions.

Chaque message publié dans un topic est soumis à un politique de rétention. De base, celle-ci est fixée à deux semaines mais il est possible de la modifier. En modifiant la politique de rétention d'une certaine manière, il est possible d'indiquer à Kafka que l'on souhaite garder les messages sur une durée de temps infini [@kreps_its_2019].

## Kafka Cluster {-}
Kafka est généralement déployé sur un cluster de machine ou de (+VM_a), mais ces machines peuvent se trouver dans des datacenter différents.

\cimg{figs/cluster.png}{scale=0.5}{Exemple de cluster et broker avec Consumer et Producer}{Source : Réalisé par Paschoud Nicolas}

Pour répondre à la propriété de tolérance aux pannes, il est possible d’ajouter un facteur de réplication sur les topics. Cela veut dire que les partitions seront répliquées n fois parmi les brokers du cluster. Une partition d’un topic peut donc se retrouver sur plusieurs brokers. Une fois la partition distribuée, un leader de partition est élu dans l’un des brokers, et elle a donc la lourde tâche de gérer les données lorsqu’une demande d’écriture ou de lecture est reçue.

\newpage

\cimg{figs/broker-partition.png}{scale=0.35}{Exemple de cluster avec deux topic et un facteur de réplication de un}{Source : Réalisé par Paschoud Nicolas}

\cimg{figs/leader-broker.png}{scale=0.35}{Exemple de cluster avec un topic ayant deux partitions. Facteur de réplication de deux et élection de la partition leader}{Source : Réalisé par Paschoud Nicolas}

Dans cet exemple, un cluster est constitué de trois broker. Dans ce cluster se trouve un topic ayant deux partitions. Ces deux partitions sont répliquées deux fois à travers les broker et l'une des deux répliques est élue partition leader.
Si l'un des broker venait à tomber, les autres membres du cluster eux continuent de tourner et de distribuer les données aux clients. Si le borker tombé contenait des partitions leader, un nouveau broker est élu leader.

### Accès au cluster depuis l'exterieur du réseau {-}
Pour se connecter au cluster depuis l'extérieur, le client doit avoir les informations des brokers du cluster.
Etant un système distribué, un leader est élu et le client qui se connecte ne sait pas sur quel borker envoyer les données. Pour connaître l'adresse du broker responsable de la partition désirée, il doit accéder à l'un des broker qui lui renverra les metadata concernant le leader [@moffatt_kafka_2019].
Les `listeners` sont des port d'écoutes sur lesquels les brokers attendent des informations provenant des autres broker du cluster.
Les `advertised.listeners` sont des listeners à disposition des clients qui souhaitent se connecter depuis l'extérieur du réseau au cluster. Ces listeners permettront aux clients de récupérer les metadatas des brokers des partitions souhaitées. Les `advertised.listeners` sont des points d'entrée et de sortie vers et depuis le cluster et offre un accès aux données depuis n'importe où.

## Publier et consommer des messages {-}
Kafka possède deux (+API_a) qui permettent d'accéder aux données du cluster : 
\begin{itemize}
\item Producer API
\item Consumer API
\end{itemize}

Producer API permet de se connecter au cluster et de produire des messages dans un topic. Le Consumer lui à l'inverse du producer, permet de récupérer les messages d'un topic.

\cimg{figs/producer-consumer.png}{scale=0.5}{Exemple de cluster avec consumer et producer pour le topic Bitcoin.}{Source : Réalisé par Paschoud Nicolas}

La méthode de traitement des données utilisée pour la consommation et production de données dans Kafka est faites à l'aide de stream de données. Cette méthode de fonctionnement est particulièrement intéressante car elle permet un transfert rapide des données sans attente. En effet, les données sont transmises en continu et la vitesse de transmission est dépendante de la vitesse de traitement effectué par la cible. Ce principe de fonctionnement est particulièrement intéressant lorsque l'on souhaite récupérer les données en temps réel. Lorsqu'une connexion avec le cluster est faites, elle peut rester ouverte et se mettre en attente de message. Dès qu'un message est publié, les données sont instantanément consommées par le client

\pagebreak