# Comment traiter les flux de données {-}

## Akka {-}
La librairie AKKA développée et maintenue par Lightbend [^0], est utilisée pour le traitement des flux de données et le déploiement d'acteurs. Cette librairie est développée en Scala et tourne sur une (+JVM_a) (il est possible de développer en Java et en Scala). Akka met à disposition des développeurs des API permettants de créer des applications hautement concurentes, distribuées et réactives.
Il y a 12 API disponibles mais seulement quelques une d'entre elles sont utiles au développement :

\begin{itemize}
\item Akka Actor
\item Akka Streams
\item Akka Cluster et Cluster Sharding
\item Alpakka Kafka
\end{itemize}

## Akka Stream {-}
Un stream est un processus qui implique le déplacement et la modification de données d’une source vers un puits. Il est comparable à une chaîne de montage ou un robot effectue une opération sur chaque élément du flux. La librairie Akka Stream offre une abstraction haut niveau de ces principes.

### Source et Puits{-}
Une source est un opérateur à une sortie qui envois des données vers les récepteurs (sink) lorsque ceux-là sont prêts. L’envoi de données vers les opérateurs (sink) s’appel downstream.

Un sink, ou un puits, est un opérateur à une entrée qui accepte et reçoit des données de l’upstream. L’upstream est connecté au downstream d’une source.

Le downstream peut être ralentit par le sink si le temps de traitement des données ne peut pas être fait dans des délais résonnables. Ce processus de ralentissement est appelé back-pressure.

### Back-pressure {-}
La back-pressure est la manière qu’ont les consommateurs (sink) d’avertir les producteurs (source) qu’ils sont disponibles. Ce processus permet aussi d’avertir le producteur qu’il n’est pas capable faire le traitement dans les délais et qu’il doit donc ralentir l'envois de données.

### Flow {-}
Un flow ou flux, possède une entrée et une sortie, et permet de connecter un upstream avec un downstream. Les données passant par le flux sont transférées de la source au puits en étant modifiées. Ces flow peuvent être connecter les uns aux autres, ils sont cummulable et cela permet de créer une "chaîne de montage" à travers laquelle les données sont traitées. Dans un exemple simple, un flow connecte une `Source` et un `Sink`.

\cimg{figs/akka/akka-flow.png}{scale=0.5}{Exemple de flow de données entre une source et un puit}{Source : Réalisé par Paschoud Nicolas}

L'ensemble des fonctionnalités offertent par la librairie, permettent de construire des applications de traitements de données performantes sans difficulté. Connecter les uns aux autres, ces outils permettent de créer des flux de données au travers des quels un ensemble de données brute est transformé. L'un des outils les plus intéressant de cette librairie permet de créer des flux de données en les déssinants sous forme de graph.

## Akka Graph {-}
Akka Graph est un outil compris dans la librairie Akka Stream. Cet outil permet de créer des graph de fonctions au travers desquels les données transitent.

\cimg{figs/akka/exemple-graph.png}{scale=0.5}{Représentation d'un graph Akka de manière visuel}{Source : akka.io \url{https://doc.akka.io/docs/akka/current/stream/stream-graphs.html}, ref. URL13}

Cet exemple visuel peut se traduire aisément en graph Akka grâce à la syntaxe offerte par la librairie.

Le code suivant représente le graph présenté précédemment.

```scala
// Entrée et sortie du graph
val in = Source(1 to 10)
val out = Sink.ignore
// Fonction représentant les embranchements du graph
val bcast = builder.add(Broadcast[Int](2))
val merge = builder.add(Merge[Int](2))
// Fonctions du graph
val f1, f2, f3, f4 = Flow[Int].map(_ + 10)
// Représentation logique du graph
in ~> f1 ~> bcast ~> f2 ~> merge ~> f3 ~> out
bcast ~> f4 ~> merge
ClosedShape
```

Les fonctions `in` et `out` sont les entrées et sorties des données. L'entrée `in` est une `Source` contenant les valeurs de 1 à 10.
La sortie `out` est un `Sink` qui n'applique aucun traitement aux données.

Les fonctions `bcast` et `merge` sont des fonctions qui permettent de séparer ou regrouper les données qui traverse le graph.
La fonction `bcast` transforme un canal en deux canaux de sorties sur lesquelles les données en entrée sont publiées.
La fonction `merge` est l'opposé de `bcast`, elle prend plusieurs flux en entrée et les transforme en un seul et unique flux de données.

Les quatre fonctions `f1`, `f2`, `f3` et `f4` ont pour rôle de modifier les données en ajoutant 10 à chacune des données qui transitent.

Toutes ces fonctions sont connectées les unes aux autres à l'aide de l'opérateur `~>` (le sens opposé existe aussi : `<~`).

Dans cet exemple le graph est dit fermer, c'est à dire qu'il n'est pas possible de le récupérer pour l'utiliser, il s'exécute de lui même sans interagir avec d'autre `Source`, `Sink` ou `Flow` externe. Il est cependant possible de transformer ces graphs en `Source`, `Sink` et `Flow`.

Cette fonctionnalité offerte par Akka Stream est à la base du fonctionnement de l'API. L'ensemble des fonctionnalités offertes sont créées à l'aide de Graph Akka.

## Akka Actor {-}
Akka Actors est un outil de la librairie Akka qui permet de programmer des acteurs. En informatique, un acteur est une primitive de la programmation concurrente. Ces acteurs sont constitués d’un état et d’un comportement et communiquent entre eux exclusivement par messages. En réponse à un message, un acteur peut effectuer un traitement, envoyer d’autre messages ou déployer d’autres acteurs.
Il faut voir une architecture d’acteurs comme une hiérarchie. Le premier acteur va vouloir distribuer son travail et le déléguer. Pour ce faire, celui-ci va déployer d’autres acteurs capables de faire ce travail. Ce processus se fera jusqu’à ce que les tâches à effectuer soient assez petites pour être faites en une seule fois. La modélisation de cette hiérarchie se fait généralement en raisonnant sur les messages qui seront traités, sur la manière dont une erreur sera gérée et quel sera le comportement normal de l’acteur.

Akka propose une abstraction de ce principe en mettant à disposition une librairie de développement de systèmes concurrents et distribués basés sur ces principes.
ActorSystem est le système qui permet de gérer les acteurs au sein d’une application. Il supervise les acteurs en gérant leur configuration, les événements, les messages non délivrés, et toute autres activité qui peut se passer au sein du système.

\newpage

## Akka Cluster et Cluster Sharding {-}
Akka Cluster est une librairie qui permet de déployer un cluster capable de gérer des acteurs. Cette librairie permet de faire abstraction d’un réseau de machines pour permettre le déploiement des acteurs sur celui-ci. Ce service permet de regrouper l'ensemble des acteurs au sein d’un cluster pour pouvoir les gérer et leur permettre de s'exécuter.
Elle permet de déployer un ensemble d’acteurs sur un grand nombre de machines afin de tester des (+IA_a).

\cimg{figs/arch-akka-cluster-details.png}{scale=0.5}{Exemple de cluster Akka avec quatre machines}{Source : Réalisé par Paschoud Nicolas}

Un cluster est constitué de noeuds. Un noeud est un membre logique d'un cluster, il peut y en avoir plusieur sur une seule machine et est défini par un tuple *hostname:port:uid*. Le cluster est un ensemble de noeuds rejoins via un système d'adhésion. Parmis cet ensemble de noeuds, un seul est désigné leader et joue le rôle de gérant. Le leader gére les transitions d'état et d'adhésion des différents acteurs du cluster.

Cluster Sharding permet de répartir les agents sur le cluster et offre la possibilité d'intéragir avec eux à l'aide de leur identifiant logique sans avoir à se soucier de leur emplacement physique. Dans ce contexte, les acteurs identifiés sont appelé entités. Les entités sont réparties sur plusieurs noeuds du cluster. Elles sont réparties mais il n'y a qu'une seule d'entre elle qui s'exécute. Lorsqu'un message est envoyé à une entité via son identifiant, le cluster se charge de délivrer le message au bon endroit, le développeur n'ayant pas connaissance de son emplacement physique. Si la machine sur laquelle une entité en cours d'exécution venait à tomber, l'une des autres entités présente sur le cluster prendra le relais.

## Alpakka Kafka {-}
Alpakka Kafka est une librairie permettant d'utiliser les fonctionnalités Akka pour accéder aux données d'un cluster Kafka. Cette librairie permet de faire abastraction de la librairie Java mise à disposition par Kafka. Avec Alpakka il est donc possible de produire et consommer des données à l'aide de `Source` et de `Sink`.

La source permet de récupérer les données d'un topic. En connecter cette source à des flow il est possible de manipuler les données.

A l'inverse de la source, le `Sink` permet de produire des données dans le cluster Kafka. Dans un scénario imaginaire, il est grâce à ces deux fonctionnalités de consommer les données d'un topic, faire un traitement afin de les produire dans un autre topic.

\cimg{figs/alpakka-exemple.png}{scale=0.5}{Exemple Alpakka}{Source : Réalisé par Paschoud Nicolas}

Les informations de connexion au cluster doivent être indiquée dans le fichier de configuration de la librairie (voir chapitre [Configuration des librairies Akka](#configuration-des-librairies-akka)).
La configuration fonctionnant par héritage, le seul paramètre qu'il est nécessaire de modifier est `bootstrap.servers`. Ce paramètre doit être rempli avec les adresses IP des instances Zookeeper.


[^0]: Lightbend : https://www.lightbend.com/

\pagebreak