# Conception de l'architecture {-}
L'ensemble des information relatant la conception de l’architecture du projet est décrite dans ce chapitre. De quelle manière les outils vus dans la phase d’analyse ont été déployés et comment ils interagissent entre eux.

Ce schéma représente le concept d'architecture souhaité. Il a été imaginé 
\cimg{figs/arch-global.png}{scale=0.4}{Concept de l'architecutre mise en place}{Source : Réalisé par Paschoud Nicolas}

On distingue sur ce schéma deux parties centrales : 
\begin{itemize}
\item Kafka Cluster
\item AKKA Cluster
\end{itemize}

La partie Kafka Cluster permettra de gérer et mettre à disposition du système les données boursières qui y auront été stockée.
La partie AKKA Cluster elle gérera les acteurs déployé sur le système. Ceux-ci seront connectés au cluster Kafka afin de récupérer les données qu'ils utilisent pour l'analyse.

## Kafka en tant que base de données {-}
L’objectif de la partie Kafka Cluster est d’utiliser la puissance de Kafka pour stocker et fournir les données boursières sous forme de stream infini de données.

Kafka est généralement utilisé comme système de message broker et n’est pas utilisé comme outil de stockage. Il est cependant fait pour stocker de manière persistante des données, il est possible de faire cela en le configurant de la bonne manière. The New York Times par exemple utilise Kafka comme base de données persistante pour y stocker les données de leurs articles. [@kreps_its_2019]

Aucune base de données comme Cassandra DB, MySQL ou encore MongoDB n'est nécessaire, Kafka rempli amplement ce rôle. Il faut cependant configurer correctement le cluster et les topics en modifiant le paramètre de persistance des données et en augmentant le facteur de réplication. L’augmentation du facteur de réplication des topics permetra d’assurer la perénnité des données en les distribuant à travers les noeuds du cluster.
Les topics font office de table comme dans une base de données. Chaque topic est à la fois la source des données d’entraînement et des données en temps réel pour le trading. Les acteurs viennent récupérer les données directement dans les topics car Kafka est très performant dans la re distribution.

Le cluster Kafka est très important car il stocke et fournit les données à tout le système.

\cimg{figs/arch-kafka-cluster.png}{scale=0.5}{Architecture représentant le cluster Kafka ainsi que les sources de données}{Source : Réalisé par Paschoud Nicolas}

Deux type de software sont connectés au cluster, Data Puller et Real Time Data. La partie Data Puller fournit les données d’entraînement des intelligences artificielles au système. La partie Real Time Data elle fournira les données boursière des actifs suivis en temps réel.

### Récupération des données {-}
Dans la partie gauche du schéma, une partie importante du système, les puller de données. Les puller permettent de récupérer les informations boursières mises à disposition par les API vus précédemment.

\cimg{figs/arch-kafka-puller.png}{scale=0.5}{Data puller du Cluster Kafka}{Source : Réalisé par Paschoud Nicolas}

Cette partie de l’architecture est connectée au cluster Kafka afin d'y envoyer les données des actifs boursiers récupéré. Cette partie est essentielle et il faut s’assurer que tout le processus de récupération soit exécuté et qu’il se passe bien.

Des WebSocket ont pour objectif de récupérer les données des actifs en temps réel. Lorsqu'une information est reçue par le WebSocket, elle est directement transformée puis envoyée au cluster Kafka pour la sauvegarde et la redistribution aux acteurs qui en ont besoin.

\cimg{figs/arch-kafka-time.png}{scale=0.5}{Schéma architectural Data puller du Cluster Kafka}{Source : Réalisé par Paschoud Nicolas}

De la même manière que les API, les websocket seront directement connectés au cluster Kafka et y enverront directement les données. Elles seront ensuite lues par les consumer des acteurs en service.

\pagebreak

## Mise à disposition des données pour les clients {-}
Les acteurs au sein du système se connectent au cluster Kafka afin de récupérer les informations qui leurs sont nécessaires. Il y a un lien direct entre les acteurs et le cluster :

\cimg{figs/arch-stream-data.png}{scale=0.5}{Schéma architectural mise à disposition de stream de données}{Source : Réalisé par Paschoud Nicolas}

Chaque acteur a en sa possession une source Kafka qui récupère les données du topic cible. Ces streams de données passent ensuite par le graph Akka que chaque acteur possède. Ces données subiront des modifications en passant par différentes fonction que le développeur aura implémentées ou des fonctions mises à disposition par l'API.

### Organisation des topics pour la sauvegarde des données boursières {-}
L'ensemble des informations récupérées par les puller et WebSocket sont stockées dans des topics dédiés au type d'informations télechargées. Dans ce cas, il y a deux types d'informations : OHLC et Order Book. Il y a cependant une spécificité pour les données OHLC. En effet, les puller récupèrent les informations sur un intervalle de temps de une minute tandis que les WebSocket eux, les récupèrent en temps réel. Il y a donc deux topics OHLC et un pour l'order book. Les topics sont organisé de la manière suivante.

| Actif              | Topic Minutes / Minutes | Topic Temps Réel| Topic Order Book |
| :----------------- | ----------------------- | --------------- | ---------------- |
| Bitcoin / Dollar   | BTCUSD_m                | BTCUSD          | BTCUSD_ob        |
| Bitcoin / Euro     | BTCEUR_m                | BTCEUR          | BTCEUR_ob        |
| Ethereum / Bitcoin | ETHBTC_m                | BTCETH          | ETHBTC_ob        |
| Ethereum / Euro    | ETHEUR_m                | ETHEUR          | ETHEUR_ob        |
| Ethereum / Dollar  | ETHUSD_m                | ETHUSD          | ETHUSD_ob        |
| EOS / Bitcoin      | EOSBTC_m                | EOSBTC          | EOSBTC_ob        |
| EOS / Dollar       | EOSUSD_m                | EOSUSD          | EOSUSD_ob        |
| Litecoin / Bitcoin | LTCBTC_m                | LTCBTC          | LTCBTC_ob        |
| Litecoin / Dollar  | LTCUSD_m                | LTCUSD          | LTCUSD_ob        |
| XRP / Bitcoin      | XRPBTC_m                | XRPBTC          | XRPBTC_ob        |
| XRP / Dollar       | XRPUSD_m                | XRPUSD          | XRPUSD_ob        |
| ZEC / Bitcoin      | ZECBTC_m                | ZECBTC          | ZECBTC_ob        |
| ZEC / Dollar       | ZECUSD_m                | ZECUSD          | ZECUSD_ob        |

: Nom des topics pour le stockage des données boursières

\textit {Source : Réalisé par Paschoud Nicolas}

Les noms des topics sont formé à l'aide du nom de l'actif. Pour les topics spéciaux, un suffix est ajouté au nom de l'actif
\begin{itemize}
\item XXXYYY\_m, le suffix \_m indique que l'intervalle de temps entre les données est de une minutes
\item XXXYYY\_ob, le suffix \_ob indique signifie Order Book
\item XXXYYY, il s'agit des données OHLC temps réel fournis par les websocket
\end{itemize}

### Publication des données boursières {-}

#### Puller de données {-}
L'ensemble des données boursières sont récupérées via des pullers ou des WebSocket.

Les pullers sont développés en Scala et sont exécutés toutes les quatre heures pour être sur de ne perdre aucune information. Avant la publication, le script s'assure de ne pas publier les données à double en supprimer les données antérieurs à la dernière date de pulling. Une fois ce filtre effectué, les données sont publié dans le topic de l'actif concerné. Les pullers récupère les données de l'historique OHLC sur l'intervalle de une minute.

#### WebSocket {-}
Les WebSockets ont initialement été développer en Scala mais après avoir rencontré de grosses difficultés ne garantissant pas la récupération continue des données, il a été décider de changer de langage par soucis de simplicité. Le problème qui a généré ce changement venait de la source elle-même, c'est à dire Kraken. Une fermeture anormale de la connexion (error code 1006) au service après 10 heures d'exécution empêchait la récupération continue des données [@noauthor_websocket_nodate]. Les librairies Scala utilisées pour le développement étant trop contraignante vis-à-vis de la gestion des erreurs empêchait de récupérer l'exception et de relancer une connexion instantanément.

Les WebSockets ont donc été développés en JavaScript à l'aide de Node.js. Les WebSockets ont pour objectifs de récupérer l'Order Book et l'OHLC en temps réel. Etant constamment connecté au service Kraken, les messages sont publiés dans les topics en temps réel. Il existe en JavaScript une librairie KafkaJS, qui permet de connecter un service développé en JavaScript à un cluster Kafka.

## Déploiement des agents sur le cluster {-}
L’ensemble des acteurs déployés sont gérés par un cluster Akka. Il a la charge de s’occuper de l’exécution de ceux-ci et de s’en débarrasser lorsque l’utilisateur n'en veut plus.

\cimg{figs/arch-akka-cluster.png}{scale=0.5}{Schéma architectural du cluster Akka}{Source : Réalisé par Paschoud Nicolas}

L’utilisateur choisit le flux de données sur lequel il souhaite faire de l’analyse et exécute son acteur. Le cluster peut accueillir des acteurs en cours d'entrainement ou de test afin de valider la stratégie, mais il accueille aussi des acteurs en exécution capable de manipuler un porte-monnaie.

Un acteur peut dépendre d’autre acteurs comme l’Actor #0 dans l’illustration. Celui-ci dépends des décisions faites par `Actor #1`, `Actor #2` et `Actor #3`. L’acteur zéro prends des décisions en se basant sur les informations et prédictions que ses frères font pour lui. Ses frères ne sont pas forcément connectés aux mêmes sources de données que l’acteur principal. Ils peuvent dépendre d’une source de données différente et permettent donc au groupe d’acteurs d’analyser le cours à des échelles différentes.

\newpage

## Synthèse {-}
Ce modèle d’architecture est construit et pensé sur la base de fonctionnement des outils présentés précédemment. C'est une architecture distribuée et il faudra des machines en réseau afin de la mettre en place. Pour cela, 10 machines virtuels ont été déployés. Trois d'entre elles serviront au déploiement du cluster Kafka, et les sept autres servent à déployer les puller de données, les websockets ainsi que le cluster Akka.

\pagebreak